@@ -1,21 +1,19 @@
 from typing import TYPE_CHECKING, Dict, List, NamedTuple, Optional
 
-from dagster import check
-from dagster.core.definitions.executor import ExecutorDefinition, default_executors
+import dagster._check as check
+from dagster.core.definitions.executor_definition import ExecutorDefinition, default_executors
 from dagster.loggers import default_loggers
-from dagster.utils.backcompat import experimental_arg_warning
 from dagster.utils.merger import merge_dicts
 
 from .config import ConfigMapping
-from .logger import LoggerDefinition
-from .resource import ResourceDefinition
+from .logger_definition import LoggerDefinition
+from .resource_definition import ResourceDefinition
 from .utils import check_valid_name
 
 DEFAULT_MODE_NAME = "default"
 
 if TYPE_CHECKING:
-    from .intermediate_storage import IntermediateStorageDefinition
-    from .partition import PartitionedConfig, PartitionSetDefinition
+    from .partition import PartitionedConfig
 
 
 class ModeDefinition(
@@ -27,7 +25,6 @@ class ModeDefinition(
             ("loggers", Dict[str, LoggerDefinition]),
             ("executor_defs", List[ExecutorDefinition]),
             ("description", Optional[str]),
-            ("intermediate_storage_defs", List["IntermediateStorageDefinition"]),
             ("config_mapping", Optional[ConfigMapping]),
             ("partitioned_config", Optional["PartitionedConfig"]),
         ],
@@ -50,11 +47,8 @@ class ModeDefinition(
             executing in this mode. By default, this will be the 'in_process' and 'multiprocess'
             executors (:py:data:`~dagster.default_executors`).
         description (Optional[str]): A human-readable description of the mode.
-        intermediate_storage_defs (Optional[List[IntermediateStorageDefinition]]): The set of intermediate storage
-            options available when executing in this mode. By default, this will be the 'in_memory'
-            and 'filesystem' system storages.
-        _config_mapping (Optional[ConfigMapping]): Experimental
-        _partitions (Optional[PartitionedConfig]): Experimental
+        _config_mapping (Optional[ConfigMapping]): Only for internal use.
+        _partitions (Optional[PartitionedConfig]): Only for internal use.
     """
 
     def __new__(
@@ -64,13 +58,10 @@ class ModeDefinition(
         logger_defs: Optional[Dict[str, LoggerDefinition]] = None,
         executor_defs: Optional[List[ExecutorDefinition]] = None,
         description: Optional[str] = None,
-        intermediate_storage_defs: Optional[List["IntermediateStorageDefinition"]] = None,
         _config_mapping: Optional[ConfigMapping] = None,
         _partitioned_config: Optional["PartitionedConfig"] = None,
     ):
-        from dagster.core.storage.system_storage import default_intermediate_storage_defs
 
-        from .intermediate_storage import IntermediateStorageDefinition
         from .partition import PartitionedConfig
 
         resource_defs = check.opt_dict_param(
@@ -90,12 +81,6 @@ class ModeDefinition(
                 {"io_manager": mem_io_manager}, resource_defs or {}
             )
 
-        if _config_mapping:
-            experimental_arg_warning("_config_mapping", "ModeDefinition.__new__")
-
-        if _partitioned_config:
-            experimental_arg_warning("_partitioned_config", "ModeDefinition.__new__")
-
         return super(ModeDefinition, cls).__new__(
             cls,
             name=check_valid_name(name) if name else DEFAULT_MODE_NAME,
@@ -106,13 +91,6 @@ class ModeDefinition(
                 )
                 or default_loggers()
             ),
-            intermediate_storage_defs=check.list_param(
-                intermediate_storage_defs
-                if intermediate_storage_defs
-                else default_intermediate_storage_defs,
-                "intermediate_storage_defs",
-                of_type=IntermediateStorageDefinition,
-            ),
             executor_defs=check.list_param(
                 executor_defs if executor_defs else default_executors,
                 "executor_defs",
@@ -129,28 +107,6 @@ class ModeDefinition(
     def resource_key_set(self):
         return frozenset(self.resource_defs.keys())
 
-    def get_intermediate_storage_def(self, name):
-        check.str_param(name, "name")
-        for intermediate_storage_def in self.intermediate_storage_defs:
-            if intermediate_storage_def.name == name:
-                return intermediate_storage_def
-
-        check.failed("{} storage definition not found".format(name))
-
-    def get_partition_set_def(self, pipeline_name: str) -> Optional["PartitionSetDefinition"]:
-        from dagster.core.definitions.partition import PartitionSetDefinition
-
-        if not self.partitioned_config:
-            return None
-
-        return PartitionSetDefinition(
-            pipeline_name=pipeline_name,
-            name=pipeline_name + "_" + self.name + "_partition_set",
-            partitions_def=self.partitioned_config.partitions_def,
-            run_config_fn_for_partition=self.partitioned_config.run_config_for_partition_fn,
-            mode=self.name,
-        )
-
     @staticmethod
     def from_resources(resources, name=None):
         check.dict_param(resources, "resources", key_type=str)
