@@ -1,15 +1,16 @@
-from __future__ import print_function
-
 import io
 import os
 import subprocess
 import sys
+import tempfile
 import time
+import uuid
 import warnings
 from contextlib import contextmanager
 
 from dagster.core.execution import poll_compute_logs, watch_orphans
-from dagster.seven import IS_WINDOWS
+from dagster.serdes.ipc import interrupt_ipc_subprocess, open_ipc_subprocess
+from dagster.seven import IS_WINDOWS, wait_for_process
 from dagster.utils import ensure_file
 
 WIN_PY36_COMPUTE_LOG_DISABLED_MSG = """\u001b[33mWARNING: Compute log capture is disabled for the current environment. Set the environment variable `PYTHONLEGACYWINDOWSSTDIO` to enable.\n\u001b[0m"""
@@ -17,7 +18,7 @@ WIN_PY36_COMPUTE_LOG_DISABLED_MSG = """\u001b[33mWARNING: Compute log capture is
 
 @contextmanager
 def redirect_to_file(stream, filepath):
-    with open(filepath, "a+", buffering=1) as file_stream:
+    with open(filepath, "a+", buffering=1, encoding="utf8") as file_stream:
         with redirect_stream(file_stream, stream):
             yield
 
@@ -33,12 +34,7 @@ def mirror_stream_to_file(stream, filepath):
 def should_disable_io_stream_redirect():
     # See https://stackoverflow.com/a/52377087
     # https://www.python.org/dev/peps/pep-0528/
-    return (
-        os.name == "nt"
-        and sys.version_info.major == 3
-        and sys.version_info.minor >= 6
-        and not os.environ.get("PYTHONLEGACYWINDOWSSTDIO")
-    )
+    return os.name == "nt" and not os.environ.get("PYTHONLEGACYWINDOWSSTDIO")
 
 
 def warn_if_compute_logs_disabled():
@@ -90,21 +86,34 @@ def execute_windows_tail(path, stream):
     poll_file = os.path.abspath(poll_compute_logs.__file__)
     stream = stream if _fileno(stream) else None
 
-    try:
-        tail_process = subprocess.Popen(
-            [sys.executable, poll_file, path, str(os.getpid())], stdout=stream
+    with tempfile.TemporaryDirectory() as temp_dir:
+        ipc_output_file = os.path.join(
+            temp_dir, "execute-windows-tail-{uuid}".format(uuid=uuid.uuid4().hex)
         )
-        yield (tail_process.pid, None)
-    finally:
-        if tail_process:
-            time.sleep(2 * poll_compute_logs.POLLING_INTERVAL)
-            tail_process.terminate()
+
+        try:
+            tail_process = open_ipc_subprocess(
+                [sys.executable, poll_file, path, str(os.getpid()), ipc_output_file], stdout=stream
+            )
+            yield (tail_process.pid, None)
+        finally:
+            if tail_process:
+                start_time = time.time()
+                while not os.path.isfile(ipc_output_file):
+                    if time.time() - start_time > 15:
+                        raise Exception("Timed out waiting for tail process to start")
+                    time.sleep(1)
+
+                # Now that we know the tail process has started, tell it to terminate once there is
+                # nothing more to output
+                interrupt_ipc_subprocess(tail_process)
+                wait_for_process(tail_process)
 
 
 @contextmanager
 def execute_posix_tail(path, stream):
     # open a subprocess to tail the file and print to stdout
-    tail_cmd = "tail -F -c +0 {}".format(path).split(" ")
+    tail_cmd = ["tail", "-F", "-c", "+0", path]
     stream = stream if _fileno(stream) else None
 
     try:
@@ -116,7 +125,12 @@ def execute_posix_tail(path, stream):
         # current process is suddenly killed)
         watcher_file = os.path.abspath(watch_orphans.__file__)
         watcher_process = subprocess.Popen(
-            [sys.executable, watcher_file, str(os.getpid()), str(tail_process.pid),]
+            [
+                sys.executable,
+                watcher_file,
+                str(os.getpid()),
+                str(tail_process.pid),
+            ]
         )
 
         yield (tail_process.pid, watcher_process.pid)
@@ -132,6 +146,7 @@ def _clean_up_subprocess(subprocess_obj):
     try:
         if subprocess_obj:
             subprocess_obj.terminate()
+            wait_for_process(subprocess_obj)
     except OSError:
         pass
 
