@@ -17,6 +17,7 @@ from typing import (
 import dagster._check as check
 from dagster.core.definitions import InputDefinition, NodeHandle, PipelineDefinition
 from dagster.core.definitions.events import AssetLineageInfo
+from dagster.core.definitions.job_definition import JobDefinition
 from dagster.core.definitions.metadata import MetadataEntry
 from dagster.core.definitions.version_strategy import ResourceVersionContext
 from dagster.core.errors import (
@@ -138,6 +139,124 @@ class StepInputSource(ABC):
         raise NotImplementedError()
 
 
+@whitelist_for_serdes
+class FromSourceAsset(
+    NamedTuple(
+        "_FromSourceAsset",
+        [
+            ("solid_handle", NodeHandle),
+            ("input_name", str),
+        ],
+    ),
+    StepInputSource,
+):
+    """
+    Load input value from an asset
+    """
+
+    def load_input_object(
+        self,
+        step_context: "StepExecutionContext",
+        input_def: InputDefinition,
+    ) -> Iterator["DagsterEvent"]:
+        from dagster.core.definitions.asset_layer import AssetOutputInfo
+        from dagster.core.events import DagsterEvent
+        from dagster.core.execution.context.output import OutputContext
+
+        input_asset_key = step_context.pipeline_def.asset_layer.asset_key_for_input(
+            self.solid_handle, input_name=self.input_name
+        )
+        assert input_asset_key is not None
+
+        input_manager_key = step_context.pipeline_def.asset_layer.io_manager_key_for_asset(
+            input_asset_key
+        )
+
+        op_config = step_context.resolved_run_config.solids.get(str(self.solid_handle))
+        config_data = op_config.inputs.get(self.input_name) if op_config else None
+
+        loader = getattr(step_context.resources, input_manager_key)
+        resources = build_resources_for_manager(input_manager_key, step_context)
+        load_input_context = step_context.for_input_manager(
+            input_def.name,
+            config_data,
+            metadata=input_def.metadata,
+            dagster_type=input_def.dagster_type,
+            resource_config=step_context.resolved_run_config.resources[input_manager_key].config,
+            resources=resources,
+            artificial_output_context=OutputContext(
+                resources=resources, asset_info=AssetOutputInfo(key=input_asset_key)
+            ),
+        )
+
+        yield from _load_input_with_input_manager(loader, load_input_context)
+
+        metadata_entries = load_input_context.consume_metadata_entries()
+
+        yield DagsterEvent.loaded_input(
+            step_context,
+            input_name=input_def.name,
+            manager_key=input_manager_key,
+            metadata_entries=[
+                entry for entry in metadata_entries if isinstance(entry, MetadataEntry)
+            ],
+        )
+
+    def compute_version(self, step_versions, pipeline_def, resolved_run_config) -> Optional[str]:
+        from ..resolve_versions import check_valid_version, resolve_config_version
+
+        op = pipeline_def.get_solid(self.solid_handle)
+        io_manager_key = op.input_def_named(self.input_name).io_manager_key
+        io_manager_def = pipeline_def.get_mode_definition(resolved_run_config.mode).resource_defs[
+            io_manager_key
+        ]
+
+        op_config = resolved_run_config.solids.get(op.name)
+        input_config = op_config.inputs.get(self.input_name)
+        resource_config = resolved_run_config.resources.get(io_manager_key).config
+
+        version_context = ResourceVersionContext(
+            resource_def=io_manager_def,
+            resource_config=resource_config,
+        )
+
+        if pipeline_def.version_strategy is not None:
+            io_manager_def_version = pipeline_def.version_strategy.get_resource_version(
+                version_context
+            )
+        else:
+            io_manager_def_version = io_manager_def.version
+
+        if io_manager_def_version is None:
+            raise DagsterInvariantViolationError(
+                f"While using memoization, version for io manager '{io_manager_def}' was "
+                "None. Please either provide a versioning strategy for your job, or provide a "
+                "version using the io_manager decorator."
+            )
+
+        check_valid_version(io_manager_def_version)
+        return join_and_hash(
+            resolve_config_version(input_config),
+            resolve_config_version(resource_config),
+            io_manager_def_version,
+        )
+
+    def required_resource_keys(self, pipeline_def: PipelineDefinition) -> Set[str]:
+        input_asset_key = pipeline_def.asset_layer.asset_key_for_input(
+            self.solid_handle, self.input_name
+        )
+        if input_asset_key is None:
+            check.failed(
+                f"Must have an asset key associated with input {self.input_name} to load it using FromSourceAsset",
+            )
+        input_manager_key = pipeline_def.asset_layer.io_manager_key_for_asset(input_asset_key)
+        if input_manager_key is None:
+            check.failed(
+                f"Must have an io_manager associated with asset {input_asset_key} to load it using FromSourceAsset"
+            )
+        return {input_manager_key}
+
+
 @whitelist_for_serdes
 class FromRootInputManager(
     NamedTuple(
@@ -183,6 +302,7 @@ class FromRootInputManager(
             ].config,
             resources=build_resources_for_manager(input_def.root_manager_key, step_context),
         )
+
         yield from _load_input_with_input_manager(loader, load_input_context)
 
         metadata_entries = load_input_context.consume_metadata_entries()
@@ -470,6 +590,7 @@ class FromConfig(
         ):
             dagster_type = self.get_associated_input_def(step_context.pipeline_def).dagster_type
             config_data = self.get_associated_config(step_context.resolved_run_config)
+
             return dagster_type.loader.construct_from_config_value(step_context, config_data)
 
     def required_resource_keys(self, pipeline_def: PipelineDefinition) -> Set[str]:
@@ -490,9 +611,51 @@ class FromConfig(
         config_data = self.get_associated_config(resolved_run_config)
         input_def = self.get_associated_input_def(pipeline_def)
         dagster_type = input_def.dagster_type
+
         return dagster_type.loader.compute_loaded_input_version(config_data)
 
 
+@whitelist_for_serdes
+class FromDirectInputValue(
+    NamedTuple(
+        "_FromDirectInputValue",
+        [("input_name", str)],
+    ),
+    StepInputSource,
+):
+    """This input source is for direct python values to be passed as inputs to ops."""
+
+    def __new__(cls, input_name: str):
+        return super(FromDirectInputValue, cls).__new__(
+            cls,
+            input_name=input_name,
+        )
+
+    def load_input_object(
+        self, step_context: "StepExecutionContext", _input_def: InputDefinition
+    ) -> Any:
+
+        pipeline_def = step_context.pipeline_def
+        if not pipeline_def.is_job:
+            raise DagsterInvariantViolationError(
+                "Using input values with pipeline API, which is unsupported."
+            )
+
+        job_def = cast(JobDefinition, pipeline_def)
+        return job_def.get_direct_input_value(self.input_name)
+
+    def required_resource_keys(self, _pipeline_def: PipelineDefinition) -> Set[str]:
+        return set()
+
+    def compute_version(
+        self,
+        step_versions: Dict[str, Optional[str]],
+        pipeline_def: PipelineDefinition,
+        resolved_run_config: ResolvedRunConfig,
+    ) -> Optional[str]:
+        return str(self.input_name)
+
+
 @whitelist_for_serdes
 class FromDefaultValue(
     NamedTuple(
@@ -591,13 +754,24 @@ class FromMultipleSources(
         step_context: "StepExecutionContext",
         input_def: InputDefinition,
     ):
-        from dagster.core.events import DagsterEvent
+        from dagster.core.events import DagsterEvent, DagsterEventType
 
         values = []
 
+        # https://github.com/dagster-io/dagster/issues/3511
+        step_output_events = [
+            record.dagster_event
+            for record in step_context.instance.all_logs(
+                step_context.run_id, of_type=DagsterEventType.STEP_OUTPUT
+            )
+        ]
+
         # some upstream steps may have skipped and we allow fan-in to continue in their absence
         source_handles_to_skip = list(
-            filter(lambda x: not step_context.can_load(x), self.step_output_handle_dependencies)
+            filter(
+                lambda x: not step_context.can_load(x, step_output_events),
+                self.step_output_handle_dependencies,
+            )
         )
 
         for inner_source in self.sources:
