@@ -12,7 +12,7 @@ from typing import (
     cast,
 )
 
-from dagster import check
+import dagster._check as check
 from dagster.core.definitions import (
     GraphDefinition,
     IPipeline,
@@ -50,10 +50,12 @@ from .compute import create_step_outputs
 from .inputs import (
     FromConfig,
     FromDefaultValue,
+    FromDirectInputValue,
     FromDynamicCollect,
     FromMultipleSources,
     FromPendingDynamicStepOutput,
     FromRootInputManager,
+    FromSourceAsset,
     FromStepOutput,
     FromUnresolvedStepOutput,
     StepInput,
@@ -173,6 +175,7 @@ class _PlanBuilder:
 
             input_source = get_root_graph_input_source(
                 plan_builder=self,
+                pipeline_def=pipeline_def,
                 input_name=input_name,
                 input_def=input_def,
             )
@@ -256,6 +259,7 @@ class _PlanBuilder:
             List[Union[StepInput, UnresolvedMappedStepInput, UnresolvedCollectStepInput]]
         ] = None,
     ):
+        asset_layer = self.pipeline.get_definition().asset_layer
         for solid in solids:
             handle = NodeHandle(solid.name, parent_handle)
 
@@ -304,7 +308,11 @@ class _PlanBuilder:
                         )
                     )
                 else:
-                    check.inst_param(step_input_source, "step_input_source", StepInputSource)
+                    check.inst_param(
+                        step_input_source,
+                        "step_input_source",
+                        StepInputSource,
+                    )
                     step_inputs.append(
                         StepInput(
                             name=input_name,
@@ -316,7 +324,9 @@ class _PlanBuilder:
             ### 2a. COMPUTE FUNCTION
             # Create and add execution plan step for the solid compute function
             if isinstance(solid.definition, SolidDefinition):
-                step_outputs = create_step_outputs(solid, handle, self.resolved_run_config)
+                step_outputs = create_step_outputs(
+                    solid, handle, self.resolved_run_config, asset_layer
+                )
 
                 if has_pending_input and has_unresolved_input:
                     check.failed("Can not have pending and unresolved step inputs")
@@ -402,12 +412,18 @@ def get_root_graph_input_source(
     plan_builder: _PlanBuilder,
     input_name: str,
     input_def: InputDefinition,
-) -> Optional[FromConfig]:
+    pipeline_def: PipelineDefinition,
+) -> Optional[Union[FromConfig, FromDirectInputValue]]:
+    from dagster.core.definitions.job_definition import get_direct_input_values_from_job
+
+    input_values = get_direct_input_values_from_job(pipeline_def)
+    if input_values and input_name in input_values:
+        return FromDirectInputValue(input_name=input_name)
 
     input_config = plan_builder.resolved_run_config.inputs
 
     if input_config and input_name in input_config:
-        return FromConfig(solid_handle=None, input_name=input_name)
+        return FromConfig(input_name=input_name, solid_handle=None)
 
     if input_def.dagster_type.is_nothing:
         return None
@@ -451,15 +467,19 @@ def get_step_input_source(
     solid_config = plan_builder.resolved_run_config.solids.get(str(handle))
 
     input_def = solid.definition.input_def_named(input_name)
+    asset_layer = plan_builder.pipeline.get_definition().asset_layer
 
     if (
-        input_def.root_manager_key
         # input is unconnected inside the current dependency structure
-        and not dependency_structure.has_deps(input_handle)
-    ):
+        not dependency_structure.has_deps(input_handle)
+        and
         #  make sure input is unconnected in the outer dependency structure too
-        if not solid.container_maps_input(input_handle.input_name):
+        not solid.container_maps_input(input_handle.input_name)
+    ):
+        if input_def.root_manager_key:
             return FromRootInputManager(solid_handle=handle, input_name=input_name)
+        elif asset_layer.asset_key_for_input(handle, input_handle.input_name):
+            return FromSourceAsset(solid_handle=handle, input_name=input_name)
 
     if dependency_structure.has_direct_dep(input_handle):
         solid_output_handle = dependency_structure.get_direct_dep(input_handle)
@@ -743,13 +763,41 @@ class ExecutionPlan(
         step_output_versions = check.opt_dict_param(
             step_output_versions, "step_output_versions", key_type=StepOutputHandle, value_type=str
         )
-        step_handles_to_execute = [StepHandle.parse_from_key(key) for key in step_keys_to_execute]
 
+        step_handles_to_validate_set: Set[StepHandleUnion] = {
+            StepHandle.parse_from_key(key) for key in step_keys_to_execute
+        }
+        step_handles_to_execute: List[StepHandleUnion] = []
         bad_keys = []
-        for handle in step_handles_to_execute:
+
+        for handle in step_handles_to_validate_set:
+
             if handle not in self.step_dict:
+                # Ok if the entire dynamic step is selected to execute.
+                # https://github.com/dagster-io/dagster/issues/8000
+                # Note: the assumption here is when the entire dynamic step is selected,
+                # the step_keys_to_execute will include both unresolved step (i.e. [?])
+                # and all the resolved steps (i.e. [0], ... [n]). Given that at this point
+                # we no longer track the parent known state (we don't know what "n" was),
+                # solely from the resolved handles, we can't tell if an entire dynamic
+                # node is being selected, so the best bet here is to check both unresolved
+                # and resolved handles exist. Examples:
+                # * `generate_subtasks, subtask[?], subtask[0], subtask[1], subtask[2]` will pass
+                # * `generate_subtasks, subtask[0], subtask[1], subtask[2]` will result in 3 bad
+                #   keys `subtask[0], subtask[1], subtask[2]`
+                if isinstance(handle, ResolvedFromDynamicStepHandle):
+                    unresolved_handle = handle.unresolved_form
+                    if (
+                        unresolved_handle in self.step_dict
+                        and unresolved_handle in step_handles_to_validate_set
+                    ):
+                        continue
+
                 bad_keys.append(handle.to_key())
 
+            # Add the handle to the ready-to-execute list once it's validated
+            step_handles_to_execute.append(handle)
+
         if bad_keys:
             raise DagsterExecutionStepNotFoundError(
                 f"Can not build subset plan from unknown step{'s' if len(bad_keys)> 1 else ''}: {', '.join(bad_keys)}",
