@@ -1,4 +1,3 @@
-import inspect
 import logging
 import logging.config
 import os
@@ -15,6 +14,7 @@ from typing import (
     Any,
     Callable,
     Dict,
+    Generic,
     Iterable,
     List,
     Mapping,
@@ -22,13 +22,14 @@ from typing import (
     Sequence,
     Set,
     Tuple,
+    TypeVar,
     Union,
     cast,
 )
 
 import yaml
 
-from dagster import check
+import dagster._check as check
 from dagster.core.definitions.events import AssetKey
 from dagster.core.definitions.pipeline_base import InMemoryPipeline
 from dagster.core.definitions.pipeline_definition import (
@@ -48,6 +49,7 @@ from dagster.core.storage.pipeline_run import (
     PipelineRun,
     PipelineRunStatsSnapshot,
     PipelineRunStatus,
+    RunPartitionData,
     RunRecord,
     RunsFilter,
     TagBucket,
@@ -80,7 +82,7 @@ if TYPE_CHECKING:
     from dagster.core.debug import DebugRunPayload
     from dagster.core.events import DagsterEvent, DagsterEventType
     from dagster.core.events.log import EventLogEntry
-    from dagster.core.execution.plan.resume_retry import ReexecutionPolicy
+    from dagster.core.execution.plan.resume_retry import ReexecutionStrategy
     from dagster.core.execution.stats import RunStepKeyStatsSnapshot
     from dagster.core.host_representation import (
         ExternalPipeline,
@@ -181,14 +183,17 @@ class InstanceType(Enum):
     EPHEMERAL = "EPHEMERAL"
 
 
-class MayHaveInstanceWeakref:
+T_DagsterInstance = TypeVar("T_DagsterInstance", bound="DagsterInstance")
+
+
+class MayHaveInstanceWeakref(Generic[T_DagsterInstance]):
     """Mixin for classes that can have a weakref back to a Dagster instance."""
 
     def __init__(self):
-        self._instance_weakref: Optional[weakref.ReferenceType["DagsterInstance"]] = None
+        self._instance_weakref: Optional[weakref.ReferenceType[T_DagsterInstance]] = None
 
     @property
-    def _instance(self) -> "DagsterInstance":
+    def _instance(self) -> T_DagsterInstance:
         instance = (
             self._instance_weakref()
             # Backcompat with custom subclasses that don't call super().__init__()
@@ -196,9 +201,9 @@ class MayHaveInstanceWeakref:
             if (hasattr(self, "_instance_weakref") and self._instance_weakref is not None)
             else None
         )
-        return cast("DagsterInstance", instance)
+        return cast(T_DagsterInstance, instance)
 
-    def register_instance(self, instance: "DagsterInstance"):
+    def register_instance(self, instance: T_DagsterInstance):
         check.invariant(
             # Backcompat with custom subclasses that don't call super().__init__()
             # in their own __init__ implementations
@@ -525,6 +530,28 @@ class DagsterInstance:
     def info_str(self) -> str:
         return yaml.dump(self.info_dict(), default_flow_style=False, sort_keys=False)
 
+    def schema_str(self) -> str:
+        def _schema_dict(alembic_version):
+            if not alembic_version:
+                return None
+            db_revision, head_revision = alembic_version
+            return {
+                "current": db_revision,
+                "latest": head_revision,
+            }
+
+        return yaml.dump(
+            {
+                "schema": {
+                    "event_log_storage": _schema_dict(self._event_storage.alembic_version()),
+                    "run_storage": _schema_dict(self._event_storage.alembic_version()),
+                    "schedule_storage": _schema_dict(self._event_storage.alembic_version()),
+                }
+            },
+            default_flow_style=False,
+            sort_keys=False,
+        )
+
     @property
     def run_storage(self) -> "RunStorage":
         return self._run_storage
@@ -750,6 +777,7 @@ class DagsterInstance:
         root_run_id=None,
         parent_run_id=None,
         solid_selection=None,
+        asset_selection=None,
         external_pipeline_origin=None,
         pipeline_code_origin=None,
     ):
@@ -767,6 +795,7 @@ class DagsterInstance:
         # solid_selection is only used to pass the user queries further down.
         check.opt_set_param(solids_to_execute, "solids_to_execute", of_type=str)
         check.opt_list_param(solid_selection, "solid_selection", of_type=str)
+        check.opt_set_param(asset_selection, "asset_selection", of_type=AssetKey)
 
         if solids_to_execute:
             if isinstance(pipeline_def, PipelineSubsetDefinition):
@@ -805,6 +834,7 @@ class DagsterInstance:
             run_config=run_config,
             mode=check.opt_str_param(mode, "mode", default=pipeline_def.get_default_mode_name()),
             solid_selection=solid_selection,
+            asset_selection=asset_selection,
             solids_to_execute=solids_to_execute,
             step_keys_to_execute=step_keys_to_execute,
             status=status,
@@ -836,6 +866,7 @@ class DagsterInstance:
         pipeline_snapshot,
         execution_plan_snapshot,
         parent_pipeline_snapshot,
+        asset_selection=None,
         solid_selection=None,
         external_pipeline_origin=None,
         pipeline_code_origin=None,
@@ -872,6 +903,7 @@ class DagsterInstance:
             run_id=run_id,
             run_config=run_config,
             mode=mode,
+            asset_selection=asset_selection,
             solid_selection=solid_selection,
             solids_to_execute=solids_to_execute,
             step_keys_to_execute=step_keys_to_execute,
@@ -984,6 +1016,7 @@ class DagsterInstance:
         pipeline_snapshot,
         execution_plan_snapshot,
         parent_pipeline_snapshot,
+        asset_selection=None,
         solid_selection=None,
         external_pipeline_origin=None,
         pipeline_code_origin=None,
@@ -994,6 +1027,7 @@ class DagsterInstance:
             run_id=run_id,
             run_config=run_config,
             mode=mode,
+            asset_selection=asset_selection,
             solid_selection=solid_selection,
             solids_to_execute=solids_to_execute,
             step_keys_to_execute=step_keys_to_execute,
@@ -1020,14 +1054,14 @@ class DagsterInstance:
         parent_run: PipelineRun,
         repo_location: "RepositoryLocation",
         external_pipeline: "ExternalPipeline",
-        policy: "ReexecutionPolicy",
+        strategy: "ReexecutionStrategy",
         extra_tags: Optional[Dict[str, Any]] = None,
         run_config: Optional[Dict[str, Any]] = None,
         mode: Optional[str] = None,
         use_parent_run_tags: bool = False,
     ) -> PipelineRun:
         from dagster.core.execution.plan.resume_retry import (
-            ReexecutionPolicy,
+            ReexecutionStrategy,
             get_retry_steps_from_parent_run,
         )
         from dagster.core.host_representation import ExternalPipeline, RepositoryLocation
@@ -1035,7 +1069,7 @@ class DagsterInstance:
         check.inst_param(parent_run, "parent_run", PipelineRun)
         check.inst_param(repo_location, "repo_location", RepositoryLocation)
         check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
-        check.inst_param(policy, "policy", ReexecutionPolicy)
+        check.inst_param(strategy, "strategy", ReexecutionStrategy)
         check.opt_dict_param(extra_tags, "extra_tags", key_type=str)
         check.opt_dict_param(run_config, "run_config", key_type=str)
         check.opt_str_param(mode, "mode")
@@ -1059,7 +1093,7 @@ class DagsterInstance:
         mode = cast(str, mode if mode is not None else parent_run.mode)
         run_config = run_config if run_config is not None else parent_run.run_config
 
-        if policy == ReexecutionPolicy.FROM_FAILURE:
+        if strategy == ReexecutionStrategy.FROM_FAILURE:
             check.invariant(
                 parent_run.status == PipelineRunStatus.FAILURE,
                 "Cannot reexecute from failure a run that is not failed",
@@ -1069,11 +1103,11 @@ class DagsterInstance:
                 self, parent_run=parent_run
             )
             tags[RESUME_RETRY_TAG] = "true"
-        elif policy == ReexecutionPolicy.ALL_STEPS:
+        elif strategy == ReexecutionStrategy.ALL_STEPS:
             step_keys_to_execute = None
             known_state = None
         else:
-            raise DagsterInvariantViolationError(f"Unknown reexecution policy: {policy}")
+            raise DagsterInvariantViolationError(f"Unknown reexecution strategy: {strategy}")
 
         external_execution_plan = repo_location.get_external_execution_plan(
             external_pipeline,
@@ -1099,6 +1133,7 @@ class DagsterInstance:
             execution_plan_snapshot=external_execution_plan.execution_plan_snapshot,
             parent_pipeline_snapshot=external_pipeline.parent_pipeline_snapshot,
             solid_selection=parent_run.solid_selection,
+            asset_selection=parent_run.asset_selection,
             external_pipeline_origin=external_pipeline.get_external_origin(),
             pipeline_code_origin=external_pipeline.get_python_origin(),
         )
@@ -1243,6 +1278,15 @@ class DagsterInstance:
     def supports_bucket_queries(self):
         return self._run_storage.supports_bucket_queries
 
+    @traced
+    def get_run_partition_data(
+        self, partition_set_name: str, job_name: str, repository_label: str
+    ) -> List[RunPartitionData]:
+        """Get run partition data for a given partitioned job."""
+        return self._run_storage.get_run_partition_data(
+            partition_set_name, job_name, repository_label
+        )
+
     def wipe(self):
         self._run_storage.wipe()
         self._event_storage.wipe()
@@ -1274,6 +1318,16 @@ class DagsterInstance:
     ):
         return self._event_storage.get_logs_for_run(run_id, of_type=of_type)
 
+    @traced
+    def get_records_for_run(
+        self,
+        run_id: str,
+        cursor: Optional[str] = None,
+        of_type: Optional[Union["DagsterEventType", Set["DagsterEventType"]]] = None,
+        limit: Optional[int] = None,
+    ):
+        return self._event_storage.get_records_for_run(run_id, cursor, of_type, limit)
+
     def watch_event_logs(self, run_id, cursor, cb):
         return self._event_storage.watch(run_id, cursor, cb)
 
@@ -1777,11 +1831,9 @@ records = instance.get_event_records(
         return run
 
     def count_resume_run_attempts(self, run_id: str):
-        from dagster.core.events import DagsterEventType
-        from dagster.daemon.monitoring import RESUME_RUN_LOG_MESSAGE
+        from dagster.daemon.monitoring import count_resume_run_attempts
 
-        events = self.all_logs(run_id, of_type=DagsterEventType.ENGINE_EVENT)
-        return len([event for event in events if event.message == RESUME_RUN_LOG_MESSAGE])
+        return count_resume_run_attempts(self, run_id)
 
     def run_will_resume(self, run_id: str):
         if not self.run_monitoring_enabled:
@@ -1970,21 +2022,6 @@ records = instance.get_event_records(
         if DagsterInstance._EXIT_STACK:
             DagsterInstance._EXIT_STACK.close()
 
-    def get_addresses_for_step_output_versions(self, step_output_versions):
-        """
-        For each given step output, finds whether an output exists with the given
-        version, and returns its address if it does.
-
-        Args:
-            step_output_versions (Dict[(str, StepOutputHandle), str]):
-                (pipeline name, step output handle) -> version.
-
-        Returns:
-            Dict[(str, StepOutputHandle), str]: (pipeline name, step output handle) -> address.
-                For each step output, an address if there is one and None otherwise.
-        """
-        return self._event_storage.get_addresses_for_step_output_versions(step_output_versions)
-
     # dagster daemon
     def add_daemon_heartbeat(self, daemon_heartbeat: "DaemonHeartbeat"):
         """Called on a regular interval by the daemon"""
