@@ -2,7 +2,7 @@ import datetime
 import logging
 from typing import TYPE_CHECKING, Any, Dict, List, NamedTuple, Optional, Union
 
-from dagster import check
+import dagster._check as check
 from dagster.core.utils import coerce_valid_log_level, make_new_run_id
 from dagster.utils.log import get_dagster_logger
 
@@ -98,13 +98,13 @@ class DagsterLoggingMetadata(
 
     def __new__(
         cls,
-        run_id: str = None,
-        pipeline_name: str = None,
-        pipeline_tags: Dict[str, str] = None,
-        step_key: str = None,
-        solid_name: str = None,
-        resource_name: str = None,
-        resource_fn_name: str = None,
+        run_id: Optional[str] = None,
+        pipeline_name: Optional[str] = None,
+        pipeline_tags: Optional[Dict[str, str]] = None,
+        step_key: Optional[str] = None,
+        solid_name: Optional[str] = None,
+        resource_name: Optional[str] = None,
+        resource_fn_name: Optional[str] = None,
     ):
         return super().__new__(
             cls,
@@ -139,22 +139,19 @@ def construct_log_string(
         if message_props.event_type_value in EVENT_TYPE_VALUE_TO_DISPLAY_STRING
         else message_props.event_type_value
     )
-    return (
-        " - ".join(
-            filter(
-                None,
-                (
-                    logging_metadata.log_source,
-                    logging_metadata.run_id,
-                    message_props.pid,
-                    logging_metadata.step_key,
-                    event_type_str,
-                    message_props.orig_message,
-                ),
-            )
+    return " - ".join(
+        filter(
+            None,
+            (
+                logging_metadata.log_source,
+                logging_metadata.run_id,
+                message_props.pid,
+                logging_metadata.step_key,
+                event_type_str,
+                message_props.orig_message,
+            ),
         )
-        + (message_props.error_str or "")
-    )
+    ) + (message_props.error_str or "")
 
 
 def get_dagster_meta_dict(
@@ -192,6 +189,7 @@ class DagsterLogHandler(logging.Handler):
         self._logging_metadata = logging_metadata
         self._loggers = loggers
         self._handlers = handlers
+        self._should_capture = True
         super().__init__()
 
     @property
@@ -242,23 +240,33 @@ class DagsterLogHandler(logging.Handler):
         multiple times, as the DagsterLogHandler will be invoked at each level of the hierarchy as
         the message is propagated. This filter prevents this from happening.
         """
-        return not isinstance(getattr(record, DAGSTER_META_KEY, None), dict)
+        return self._should_capture and not isinstance(
+            getattr(record, DAGSTER_META_KEY, None), dict
+        )
 
     def emit(self, record: logging.LogRecord):
         """For any received record, add Dagster metadata, and have handlers handle it"""
-        dagster_record = self._convert_record(record)
-        # built-in handlers
-        for handler in self._handlers:
-            if dagster_record.levelno >= handler.level:
-                handler.handle(dagster_record)
-        # user-defined @loggers
-        for logger in self._loggers:
-            logger.log(
-                dagster_record.levelno,
-                dagster_record.msg,
-                exc_info=dagster_record.exc_info,
-                extra=self._extract_extra(record),
-            )
+
+        try:
+            # to prevent the potential for infinite loops in which a handler produces log messages
+            # which are then captured and then handled by that same handler (etc.), do not capture
+            # any log messages while one is currently being emitted
+            self._should_capture = False
+            dagster_record = self._convert_record(record)
+            # built-in handlers
+            for handler in self._handlers:
+                if dagster_record.levelno >= handler.level:
+                    handler.handle(dagster_record)
+            # user-defined @loggers
+            for logger in self._loggers:
+                logger.log(
+                    dagster_record.levelno,
+                    dagster_record.msg,
+                    exc_info=dagster_record.exc_info,
+                    extra=self._extract_extra(record),
+                )
+        finally:
+            self._should_capture = True
 
 
 class DagsterLogManager(logging.Logger):
@@ -288,7 +296,7 @@ class DagsterLogManager(logging.Logger):
         self,
         dagster_handler: DagsterLogHandler,
         level: int = logging.NOTSET,
-        managed_loggers: List[logging.Logger] = None,
+        managed_loggers: Optional[List[logging.Logger]] = None,
     ):
         super().__init__(name="dagster", level=coerce_valid_log_level(level))
         self._managed_loggers = check.opt_list_param(
@@ -301,7 +309,7 @@ class DagsterLogManager(logging.Logger):
     def create(
         cls,
         loggers: List[logging.Logger],
-        handlers: List[logging.Handler] = None,
+        handlers: Optional[List[logging.Handler]] = None,
         instance: Optional["DagsterInstance"] = None,
         pipeline_run: Optional["PipelineRun"] = None,
     ) -> "DagsterLogManager":
