@@ -4,7 +4,6 @@ import os
 import sqlite3
 import threading
 import time
-import warnings
 from collections import defaultdict
 from contextlib import contextmanager
 from typing import Iterable, Optional
@@ -20,13 +19,12 @@ import dagster.seven as seven
 from dagster.config.source import StringSource
 from dagster.core.events import DagsterEventType
 from dagster.core.events.log import EventLogEntry
-from dagster.core.storage.event_log.base import EventLogRecord, EventRecordsFilter
+from dagster.core.storage.event_log.base import EventLogCursor, EventLogRecord, EventRecordsFilter
 from dagster.core.storage.pipeline_run import PipelineRunStatus, RunsFilter
 from dagster.core.storage.sql import (
     check_alembic_revision,
     create_engine,
     get_alembic_config,
-    handle_schema_errors,
     run_alembic_upgrade,
     stamp_alembic_rev,
 )
@@ -201,8 +199,7 @@ class SqliteEventLogStorage(SqlEventLogStorage, ConfigurableClass):
             conn = engine.connect()
 
             try:
-                with handle_schema_errors(conn, get_alembic_config(__file__)):
-                    yield conn
+                yield conn
             finally:
                 conn.close()
             engine.dispose()
@@ -279,15 +276,16 @@ class SqliteEventLogStorage(SqlEventLogStorage, ConfigurableClass):
         else:
             asset_details = None
 
-        if not event_records_filter or not (
-            isinstance(event_records_filter.after_cursor, RunShardedEventsCursor)
+        if (
+            event_records_filter
+            and event_records_filter.after_cursor != None
+            and not isinstance(event_records_filter.after_cursor, RunShardedEventsCursor)
         ):
-            warnings.warn(
+            raise Exception(
                 """
-                Called `get_event_records` on a run-sharded event log storage with a query that
-                is not run aware (e.g. not using a RunShardedEventsCursor).  This likely has poor
-                performance characteristics.  Consider adding a RunShardedEventsCursor to your query
-                or switching your instance configuration to use a non-run sharded event log storage
+                Called `get_event_records` on a run-sharded event log storage with a cursor that
+                is not run-aware. Add a RunShardedEventsCursor to your query filter
+                or switch your instance configuration to use a non-run-sharded event log storage
                 (e.g. PostgresEventLogStorage, ConsolidatedSqliteEventLogStorage)
             """
             )
@@ -384,12 +382,12 @@ class SqliteEventLogStorage(SqlEventLogStorage, ConfigurableClass):
         super(SqliteEventLogStorage, self).wipe_asset(asset_key)
         self._delete_mirrored_events_for_asset_key(asset_key)
 
-    def watch(self, run_id, start_cursor, callback):
+    def watch(self, run_id, cursor, callback):
         if not self._obs:
             self._obs = Observer()
             self._obs.start()
 
-        watchdog = SqliteEventLogStorageWatchdog(self, run_id, callback, start_cursor)
+        watchdog = SqliteEventLogStorageWatchdog(self, run_id, callback, cursor)
         self._watchers[run_id][callback] = (
             watchdog,
             self._obs.schedule(watchdog, self._base_dir, True),
@@ -406,25 +404,33 @@ class SqliteEventLogStorage(SqlEventLogStorage, ConfigurableClass):
             self._obs.stop()
             self._obs.join(timeout=15)
 
+    def alembic_version(self):
+        alembic_config = get_alembic_config(__file__)
+        with self.index_connection() as conn:
+            return check_alembic_revision(alembic_config, conn)
+
 
 class SqliteEventLogStorageWatchdog(PatternMatchingEventHandler):
-    def __init__(self, event_log_storage, run_id, callback, start_cursor, **kwargs):
+    def __init__(self, event_log_storage, run_id, callback, cursor, **kwargs):
         self._event_log_storage = check.inst_param(
             event_log_storage, "event_log_storage", SqliteEventLogStorage
         )
         self._run_id = check.str_param(run_id, "run_id")
         self._cb = check.callable_param(callback, "callback")
         self._log_path = event_log_storage.path_for_shard(run_id)
-        self._cursor = start_cursor if start_cursor is not None else -1
+        self._cursor = cursor
         super(SqliteEventLogStorageWatchdog, self).__init__(patterns=[self._log_path], **kwargs)
 
     def _process_log(self):
-        events = self._event_log_storage.get_logs_for_run(self._run_id, self._cursor)
-        self._cursor += len(events)
-        for event in events:
+        connection = self._event_log_storage.get_records_for_run(self._run_id, self._cursor)
+        if connection.cursor:
+            self._cursor = connection.cursor
+        for record in connection.records:
             status = None
             try:
-                status = self._cb(event)
+                status = self._cb(
+                    record.event_log_entry, str(EventLogCursor.from_storage_id(record.storage_id))
+                )
             except Exception:
                 logging.exception("Exception in callback for event watch on run %s.", self._run_id)
 
