@@ -1,9 +1,10 @@
 import warnings
 from datetime import datetime
 from enum import Enum
-from typing import TYPE_CHECKING, AbstractSet, Any, Dict, List, NamedTuple, Optional, Type
+from typing import TYPE_CHECKING, Any, Dict, FrozenSet, List, Mapping, NamedTuple, Optional, Type
 
-from dagster import check
+import dagster._check as check
+from dagster.core.definitions.events import AssetKey
 from dagster.core.origin import PipelinePythonOrigin
 from dagster.core.storage.tags import PARENT_RUN_ID_TAG, ROOT_RUN_ID_TAG
 from dagster.core.utils import make_new_run_id
@@ -22,6 +23,7 @@ from .tags import (
     BACKFILL_ID_TAG,
     PARTITION_NAME_TAG,
     PARTITION_SET_TAG,
+    REPOSITORY_LABEL_TAG,
     RESUME_RETRY_TAG,
     SCHEDULE_NAME_TAG,
     SENSOR_NAME_TAG,
@@ -166,6 +168,7 @@ def pipeline_run_from_storage(
     run_id=None,
     run_config=None,
     mode=None,
+    asset_selection=None,
     solid_selection=None,
     solids_to_execute=None,
     step_keys_to_execute=None,
@@ -195,6 +198,7 @@ def pipeline_run_from_storage(
     # * added solid_subset
     # * renamed solid_subset -> solid_selection, added solids_to_execute
     # * renamed environment_dict -> run_config
+    # * added asset_selection
 
     # back compat for environment dict => run_config
     if environment_dict:
@@ -253,6 +257,7 @@ def pipeline_run_from_storage(
         run_id=run_id,
         run_config=run_config,
         mode=mode,
+        asset_selection=asset_selection,
         solid_selection=solid_selection,
         solids_to_execute=solids_to_execute,
         step_keys_to_execute=step_keys_to_execute,
@@ -273,10 +278,11 @@ class PipelineRun(
         [
             ("pipeline_name", str),
             ("run_id", str),
-            ("run_config", Dict[str, object]),
+            ("run_config", Mapping[str, object]),
             ("mode", Optional[str]),
+            ("asset_selection", Optional[FrozenSet[AssetKey]]),
             ("solid_selection", Optional[List[str]]),
-            ("solids_to_execute", Optional[AbstractSet[str]]),
+            ("solids_to_execute", Optional[FrozenSet[str]]),
             ("step_keys_to_execute", Optional[List[str]]),
             ("status", PipelineRunStatus),
             ("tags", Dict[str, str]),
@@ -297,10 +303,11 @@ class PipelineRun(
         cls,
         pipeline_name: str,
         run_id: Optional[str] = None,
-        run_config: Optional[Dict[str, object]] = None,
+        run_config: Optional[Mapping[str, object]] = None,
         mode: Optional[str] = None,
+        asset_selection: Optional[FrozenSet[AssetKey]] = None,
         solid_selection: Optional[List[str]] = None,
-        solids_to_execute: Optional[AbstractSet[str]] = None,
+        solids_to_execute: Optional[FrozenSet[str]] = None,
         step_keys_to_execute: Optional[List[str]] = None,
         status: Optional[PipelineRunStatus] = None,
         tags: Optional[Dict[str, str]] = None,
@@ -330,6 +337,10 @@ class PipelineRun(
         )
         check.opt_nullable_list_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
 
+        asset_selection = check.opt_nullable_set_param(
+            asset_selection, "asset_selection", of_type=AssetKey
+        )
+
         # Placing this with the other imports causes a cyclic import
         # https://github.com/dagster-io/dagster/issues/3181
         from dagster.core.host_representation.origin import ExternalPipelineOrigin
@@ -349,9 +360,10 @@ class PipelineRun(
             cls,
             pipeline_name=check.str_param(pipeline_name, "pipeline_name"),
             run_id=check.str_param(run_id, "run_id"),
-            run_config=check.opt_dict_param(run_config, "run_config", key_type=str),
+            run_config=check.opt_mapping_param(run_config, "run_config", key_type=str),
             mode=check.opt_str_param(mode, "mode"),
             solid_selection=solid_selection,
+            asset_selection=asset_selection,
             solids_to_execute=solids_to_execute,
             step_keys_to_execute=step_keys_to_execute,
             status=check.opt_inst_param(
@@ -398,6 +410,20 @@ class PipelineRun(
     def get_parent_run_id(self):
         return self.tags.get(PARENT_RUN_ID_TAG)
 
+    def tags_for_storage(self):
+        repository_tags = {}
+        if self.external_pipeline_origin:
+            # tag the run with a label containing the repository name / location name, to allow for
+            # per-repository filtering of runs from dagit.
+            repository_tags[
+                REPOSITORY_LABEL_TAG
+            ] = self.external_pipeline_origin.external_repository_origin.get_label()
+
+        if not self.tags:
+            return repository_tags
+
+        return {**repository_tags, **self.tags}
+
     @property
     def is_finished(self):
         return (
@@ -595,6 +621,37 @@ class RunRecord(
         )
 
 
+@whitelist_for_serdes
+class RunPartitionData(
+    NamedTuple(
+        "_RunPartitionData",
+        [
+            ("run_id", str),
+            ("partition", str),
+            ("status", DagsterRunStatus),
+            ("start_time", Optional[float]),
+            ("end_time", Optional[float]),
+        ],
+    )
+):
+    def __new__(
+        cls,
+        run_id: str,
+        partition: str,
+        status: DagsterRunStatus,
+        start_time: Optional[float],
+        end_time: Optional[float],
+    ):
+        return super(RunPartitionData, cls).__new__(
+            cls,
+            run_id=check.str_param(run_id, "run_id"),
+            partition=check.str_param(partition, "partition"),
+            status=check.inst_param(status, "status", DagsterRunStatus),
+            start_time=check.opt_inst(start_time, float),
+            end_time=check.opt_inst(end_time, float),
+        )
+
+
 ###################################################################################################
 # GRAVEYARD
 #
