@@ -3,7 +3,7 @@ from typing import Callable, Union
 
 import pendulum
 
-from dagster import check
+import dagster._check as check
 from dagster.core.definitions.partition import Partition, PartitionSetDefinition
 from dagster.core.definitions.run_request import SkipReason
 from dagster.core.definitions.schedule_definition import ScheduleEvaluationContext
